from django.conf import settings
from django.template.loader import render_to_string, TemplateDoesNotExist

APP_BASE_DIR = settings.BASE_DIR + '/hx_toolkit/'
STATIC_OUTPUT_DIR = APP_BASE_DIR + 'static/hx_toolkit_output/'
ARTICLE_OUTPUT_DIR = APP_BASE_DIR + 'templates/hx_toolkit_output/'
ARTICLE_BY_SLUG_DIR = ARTICLE_OUTPUT_DIR + "by_id/"
ARTICLE_BY_WEEK_DIR = ARTICLE_OUTPUT_DIR + "weekly/"
SUMMARY_LINKS_DIR = ARTICLE_OUTPUT_DIR + "summary/"


def get_article_links_by_category():
    category_data = {'know-yourself': _get_category_by_id('know-yourself'),
                     'make-your-way': _get_category_by_id('make-your-way'),
                     'make-sense': _get_category_by_id('make-sense'),
                     'know-world': _get_category_by_id('know-world')
                     }
    return category_data


def _get_category_by_id(id):
    path = 'hx_toolkit_output/summary/' + id + ".html"
    try:
        return render_to_string(path)
    except TemplateDoesNotExist:
        return None


def get_article_by_id(article_id):
    article_path = ARTICLE_OUTPUT_DIR + article_id + ".html"

    try:
        with open(article_path, 'r') as article_file:
            article_data = article_file.read()
            return article_data
    except IOError:
        return None


def get_rendered_article_by_id(article_id):
    article_file = "hx_toolkit_output/by_id/" + article_id + ".html"
    return render_to_string(article_file)


def get_article_by_phase_quarter_week(phase, quarter, week):
    week = str(week)
    root = "hx_toolkit_output/weekly/"

    path = root + phase + "/" + quarter + "/" + week + ".html"

    try:
        return render_to_string(path)
    except TemplateDoesNotExist:
        return None



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-12-19 19:31
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('hx_toolkit', '0003_auto_20171215_0313'),
    ]

    operations = [
        migrations.AddField(
            model_name='article',
            name='image_credit',
            field=models.TextField(blank=True, null=True),
        ),
    ]



from django.core.management.base import BaseCommand, CommandError
from hx_toolkit.db_dao import get_article_by_slug, create_article_data
from hx_toolkit.file_dao import STATIC_OUTPUT_DIR, ARTICLE_OUTPUT_DIR, \
    ARTICLE_BY_SLUG_DIR, ARTICLE_BY_WEEK_DIR, SUMMARY_LINKS_DIR
from hx_toolkit.views import render_article_html, render_category_links
from hx_toolkit.models import Article
import json
import os
import shutil
from PIL import Image


class Command(BaseCommand):
    IMAGE_OUTPUT_DIR = STATIC_OUTPUT_DIR + 'images/'
    IMAGE_REL_DIR = 'hx_toolkit_output/images/'
    IMAGE_WIDTHS = [480, 650, 780, 1000]

    def _render_articles(self, articles):
        load_static_string = "{% load static %}"
        articles_by_category = {}

        for article in articles:
            srcset_paths = {}
            for width in self.IMAGE_WIDTHS:
                path = self._move_image(article.image.path, width)
                image_static_string = self._get_static_string(path)
                srcset_paths[width] = image_static_string

            setattr(article, 'image_srcset_paths', srcset_paths)
            setattr(article, 'load_static_string', load_static_string)

            cs = article.category.slug
            articles_by_category.setdefault(cs, [])

            articles_by_category[cs].append({'slug': article.slug,
                                             'title': article.title})

            article_long_html = render_article_html(article)
            article_short_html = render_article_html(article, True)

            # save article by slug
            article_path = ARTICLE_BY_SLUG_DIR + article.get_article_filename()
            self._save_html(article_long_html, article_path)

            # save article by week path
            if article.phase:
                week_path = ARTICLE_BY_WEEK_DIR + \
                            article.phase + "/" + \
                            article.quarter + "/"
                if not os.path.exists(week_path):
                    os.makedirs(week_path)
                week_path += str(article.week) + ".html"
                self._save_html(article_short_html, week_path)

        for category in articles_by_category:
            links = articles_by_category[category]
            rendered_links = render_category_links(links)
            self._save_html(rendered_links, SUMMARY_LINKS_DIR +
                            category + ".html")

    def _save_html(self, html, path):
        with open(path, 'w+') as article_file:
            article_file.write(html.encode('utf-8'))

    def handle(self, *args, **options):
        # Ensure directories exist and are empty
        self._create_or_empty_dir(STATIC_OUTPUT_DIR)
        self._create_or_empty_dir(self.IMAGE_OUTPUT_DIR)
        self._create_or_empty_dir(ARTICLE_OUTPUT_DIR)
        self._create_or_empty_dir(ARTICLE_BY_SLUG_DIR)
        self._create_or_empty_dir(ARTICLE_BY_WEEK_DIR)
        self._create_or_empty_dir(SUMMARY_LINKS_DIR)

        """
        Article File Structure
        templates/hx_toolkit_output
            /by_slug
                [slug].html -Flat list of all -long articles
            /categories/
                [category_slug].html -Category card HTML, list of article links
            /weekly/
                /[phase]
                    /[quarter]
                        [week].html -Short version of weekly articles
        """
        articles = Article.objects.all()
        self._render_articles(articles)

    def _move_image(self, image_path, width):
        filename = os.path.basename(image_path)

        resized_image = self._resize_image(image_path, width)

        file, ext = os.path.splitext(filename)
        resized_filename = file + "_" + str(width) + ext

        new_path = self.IMAGE_OUTPUT_DIR + resized_filename
        resized_image.save(new_path)

        rel_path = self.IMAGE_REL_DIR + resized_filename

        return rel_path

    def _get_static_string(self, image_path):
        string = "{% static '" + image_path + "' %}"
        return string

    def _create_or_empty_dir(self, dir_path):
        try:
            shutil.rmtree(dir_path)
        except OSError:
            pass
        os.makedirs(dir_path)

    def _resize_image(self, image_path, width):
        image = Image.open(image_path)
        width_percent = (width / float(image.size[0]))
        height = int((float(image.size[1]) * float(width_percent)))
        image = image.resize((width, height), Image.ANTIALIAS)

        return image



from hx_toolkit.models import Article, Category


def get_articles_by_category(category_slug):
    articles = Article.objects.filter(category__slug=category_slug)
    print len(articles)


def get_article_by_slug(article_slug):
    article = Article.objects.get(slug=article_slug)

    return article


def create_article_data():
    articles_by_category = {}
    categories = Category.objects.all()

    for category in categories:
        category_articles = Article.objects.filter(category=category)
        article_data = []
        for article in category_articles:
            article_data.append(article.article_title_data())
        if article_data:
            articles_by_category[category.slug] = article_data

    articles_by_time = {}

    phases = Article.PHASE_CHOICES
    quarters = Article.QUARTER_CHOICES

    for phase in phases:
        article_by_quarter = {}
        for quarter in quarters:
            quarter_articles = Article.objects.filter(phase=phase[0],
                                                      quarter=quarter[0])
            article_by_week = {}
            for article in quarter_articles:
                article_by_week[article.week] = article.article_title_data()
            if len(article_by_week):
                article_by_quarter.setdefault(quarter[0], {})
                article_by_quarter[quarter[0]] = article_by_week
        if len(article_by_quarter):
            articles_by_time.setdefault(phase[0], {})
            articles_by_time[phase[0]] = article_by_quarter

    return {'category': articles_by_category,
            'time': articles_by_time
            }



from django.db import models


class ResourceLink(models.Model):
    url = models.URLField()
    title = models.CharField(max_length=255)

    def __unicode__(self):
        return self.title


class Category(models.Model):
    title = models.CharField(max_length=255)
    slug = models.SlugField(unique=True)

    def __unicode__(self):
        return self.title

    class Meta:
        verbose_name_plural = "Categories"


class Article(models.Model):
    PHASE_CHOICES = (
        ('A', 'Phase A'),
        ('B', 'Phase B'),
        ('C', 'Phase C'),
    )
    QUARTER_CHOICES = (
        ('autumn', 'Autumn'),
        ('winter', 'Winter'),
        ('spring', 'Spring'),
        ('summer', 'Summer')
    )

    title = models.CharField(max_length=255)
    short_body = models.TextField(blank=True,
                                  null=True)
    long_body = models.TextField()

    image = models.ImageField(upload_to='hx_toolkit_images')
    image_credit = models.TextField(blank=True,
                                    null=True)
    image_alt_text = models.TextField(blank=True,
                                      null=True)

    category = models.ForeignKey(Category,
                                 on_delete=models.PROTECT)

    slug = models.SlugField(unique=True)

    # data needed for time based card view
    phase = models.CharField(choices=PHASE_CHOICES,
                             max_length=1,
                             blank=True,
                             null=True)
    quarter = models.CharField(choices=QUARTER_CHOICES,
                               max_length=6,
                               blank=True,
                               null=True)

    # range btwn -2 and 14 weeks
    week = models.IntegerField(blank=True,
                               null=True)
    related_articles = models.ManyToManyField("self",
                                              related_name='related_articles',
                                              blank=True)
    related_links = models.ManyToManyField(ResourceLink,
                                           blank=True)

    def __unicode__(self):
        return self.title

    def article_title_data(self):
        data = {
            'title': self.title,
            'slug': self.slug
        }
        return data

    def get_article_filename(self, is_short=False):
        return self.slug + ".html"
